/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by client-gen. DO NOT EDIT.

package v1

import (
	v1 "github.com/openshift/cluster-version-operator/pkg/apis/operatorstatus.openshift.io/v1"
	scheme "github.com/openshift/cluster-version-operator/pkg/generated/clientset/versioned/scheme"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// OperatorStatusesGetter has a method to return a OperatorStatusInterface.
// A group's client should implement this interface.
type OperatorStatusesGetter interface {
	OperatorStatuses(namespace string) OperatorStatusInterface
}

// OperatorStatusInterface has methods to work with OperatorStatus resources.
type OperatorStatusInterface interface {
	Create(*v1.OperatorStatus) (*v1.OperatorStatus, error)
	Update(*v1.OperatorStatus) (*v1.OperatorStatus, error)
	Delete(name string, options *metav1.DeleteOptions) error
	DeleteCollection(options *metav1.DeleteOptions, listOptions metav1.ListOptions) error
	Get(name string, options metav1.GetOptions) (*v1.OperatorStatus, error)
	List(opts metav1.ListOptions) (*v1.OperatorStatusList, error)
	Watch(opts metav1.ListOptions) (watch.Interface, error)
	Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.OperatorStatus, err error)
	OperatorStatusExpansion
}

// operatorStatuses implements OperatorStatusInterface
type operatorStatuses struct {
	client rest.Interface
	ns     string
}

// newOperatorStatuses returns a OperatorStatuses
func newOperatorStatuses(c *OperatorstatusV1Client, namespace string) *operatorStatuses {
	return &operatorStatuses{
		client: c.RESTClient(),
		ns:     namespace,
	}
}

// Get takes name of the operatorStatus, and returns the corresponding operatorStatus object, and an error if there is any.
func (c *operatorStatuses) Get(name string, options metav1.GetOptions) (result *v1.OperatorStatus, err error) {
	result = &v1.OperatorStatus{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("operatorstatuses").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do().
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of OperatorStatuses that match those selectors.
func (c *operatorStatuses) List(opts metav1.ListOptions) (result *v1.OperatorStatusList, err error) {
	result = &v1.OperatorStatusList{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("operatorstatuses").
		VersionedParams(&opts, scheme.ParameterCodec).
		Do().
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested operatorStatuses.
func (c *operatorStatuses) Watch(opts metav1.ListOptions) (watch.Interface, error) {
	opts.Watch = true
	return c.client.Get().
		Namespace(c.ns).
		Resource("operatorstatuses").
		VersionedParams(&opts, scheme.ParameterCodec).
		Watch()
}

// Create takes the representation of a operatorStatus and creates it.  Returns the server's representation of the operatorStatus, and an error, if there is any.
func (c *operatorStatuses) Create(operatorStatus *v1.OperatorStatus) (result *v1.OperatorStatus, err error) {
	result = &v1.OperatorStatus{}
	err = c.client.Post().
		Namespace(c.ns).
		Resource("operatorstatuses").
		Body(operatorStatus).
		Do().
		Into(result)
	return
}

// Update takes the representation of a operatorStatus and updates it. Returns the server's representation of the operatorStatus, and an error, if there is any.
func (c *operatorStatuses) Update(operatorStatus *v1.OperatorStatus) (result *v1.OperatorStatus, err error) {
	result = &v1.OperatorStatus{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("operatorstatuses").
		Name(operatorStatus.Name).
		Body(operatorStatus).
		Do().
		Into(result)
	return
}

// Delete takes name of the operatorStatus and deletes it. Returns an error if one occurs.
func (c *operatorStatuses) Delete(name string, options *metav1.DeleteOptions) error {
	return c.client.Delete().
		Namespace(c.ns).
		Resource("operatorstatuses").
		Name(name).
		Body(options).
		Do().
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *operatorStatuses) DeleteCollection(options *metav1.DeleteOptions, listOptions metav1.ListOptions) error {
	return c.client.Delete().
		Namespace(c.ns).
		Resource("operatorstatuses").
		VersionedParams(&listOptions, scheme.ParameterCodec).
		Body(options).
		Do().
		Error()
}

// Patch applies the patch and returns the patched operatorStatus.
func (c *operatorStatuses) Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.OperatorStatus, err error) {
	result = &v1.OperatorStatus{}
	err = c.client.Patch(pt).
		Namespace(c.ns).
		Resource("operatorstatuses").
		SubResource(subresources...).
		Name(name).
		Body(data).
		Do().
		Into(result)
	return
}
